searchData={"items":[{"type":"module","title":"Numscriptex","doc":"NumscriptEx is a library that allows its users to check and run [Numscripts](https://docs.formance.com/numscript/)\nvia Elixir.\n\nWant to check if your script is valid and ready to go? Use the `check/1` function.\nAlready checked the script and want to execute him? Use the `run/2` function.","ref":"Numscriptex.html"},{"type":"function","title":"Numscriptex.check/1","doc":"To use `check/1` you just need to pass your numscript as its argument.\nEx:\n\n```elixir\niex> script = \"send [USD/2 100] (source = @foo destination = @bar)\"\niex> Numscriptex.check(script)\n{:ok, %{script: script}}\n```\n\nIt could also return some warnings, infos or hints inside the map","ref":"Numscriptex.html#check/1"},{"type":"type","title":"Numscriptex.check_result/0","doc":"","ref":"Numscriptex.html#t:check_result/0"},{"type":"type","title":"Numscriptex.errors/0","doc":"","ref":"Numscriptex.html#t:errors/0"},{"type":"function","title":"Numscriptex.run/2","doc":"To use `run/2` your first argument must be your script, and the second must\nbe a `%Numscriptex.Run{}` (go to `Numscriptex.Run` module to see more) struct.\nEx:\n\n```elixir\niex> script = \"send [USD/2 100] (source = @foo destination = @bar)\"\n...> balances = %{\"foo\" => %{\"USD/2\" => 500, \"EUR/2\" => 300}}\n...>\n...> struct =\n...> Numscriptex.Run.new()\n...> |> Numscriptex.Run.put!(:balances, balances)\n...> |> Numscriptex.Run.put!(:metadata, %{})\n...> |> Numscriptex.Run.put!(:variables, %{})\n...>\n...> Numscriptex.run(script, struct)\n```","ref":"Numscriptex.html#run/2"},{"type":"type","title":"Numscriptex.run_result/0","doc":"","ref":"Numscriptex.html#t:run_result/0"},{"type":"function","title":"Numscriptex.version/0","doc":"`version/0` simply shows a map with both [Numscript-WASM](https://github.com/PagoPlus/numscript-wasm) and NumscriptEx versions. Ex:\n\n```elixir\niex> Numscriptex.version()\n%{numscriptex: \"v0.2.5\", numscript_wasm: \"v0.0.2\"}\n```","ref":"Numscriptex.html#version/0"},{"type":"module","title":"Numscriptex.AssetsManager","doc":"`Numscriptex.AssetsManager` is responsible for ensuring that the Numscriptex\nlibrary have all it needs (i.e. a WASM binary file that is used to check and run Numscripts)\nto run correctly at compile time.","ref":"Numscriptex.AssetsManager.html"},{"type":"function","title":"Numscriptex.AssetsManager.binary_path/0","doc":"","ref":"Numscriptex.AssetsManager.html#binary_path/0"},{"type":"macro","title":"Numscriptex.AssetsManager.ensure_wasm_binary_is_valid/0","doc":"","ref":"Numscriptex.AssetsManager.html#ensure_wasm_binary_is_valid/0"},{"type":"function","title":"Numscriptex.AssetsManager.hash_wasm_binary/0","doc":"","ref":"Numscriptex.AssetsManager.html#hash_wasm_binary/0"},{"type":"module","title":"Numscriptex.Balance","doc":"`Numscriptex.Balances` is responsible for building the account's final balance\nafter running your [Numscript](https://docs.formance.com/numscript/), so you\ncan see the results of all transactions.","ref":"Numscriptex.Balance.html"},{"type":"function","title":"Numscriptex.Balance.put/2","doc":"Receives the account assets (balance field from `%Numscriptex.Run{}`), and the\npostings that are generated after running the numscript transaction.\n\nThe result will be a map contaning the initial and final balances of each\naccount assets. Ex:\n\n```elixir\niex> account_assets = %{\n...>     \"foo\" => %{\n...>     \"USD/2\" => 500,\n...>     \"EUR/2\" => 300\n...>   }\n...> }\n...>\n...> postings = [\n...>   %{\n...>     \"amount\" => 100,\n...>     \"asset\" => \"USD/2\",\n...>     \"destination\" => \"bar\",\n...>     \"source\" => \"foo\"\n...>   }\n...> ]\n...>\n...> Numscriptex.Balance.put(account_assets, postings)\n[\n  %Numscriptex.Balance{\n    account: \"foo\",\n    asset: \"EUR/2\",\n    final_balance: 300,\n    decimal_final_balance: 3.0,\n    initial_balance: 300,\n    decimal_initial_balance: 3.0\n  },\n  %Numscriptex.Balance{\n    account: \"foo\",\n    asset: \"USD/2\",\n    final_balance: 400,\n    decimal_final_balance: 4.0,\n    initial_balance: 500,\n    decimal_initial_balance: 5.0\n  },\n  %Numscriptex.Balance{\n    account: \"bar\",\n    asset: \"USD/2\",\n    final_balance: 100,\n    decimal_final_balance: 1.0,\n    initial_balance: 0,\n    decimal_initial_balance: 0.0\n  }\n]\n```","ref":"Numscriptex.Balance.html#put/2"},{"type":"type","title":"Numscriptex.Balance.t/0","doc":"Type that represents `Numscriptex.Balance` struct.","ref":"Numscriptex.Balance.html#t:t/0"},{"type":"type","title":"Fields - Numscriptex.Balance.t/0","doc":"* `:account` the account name\n* `:asset` the asset were the transaction was made\n* `:final_balance` balance after the transactions (integer)\n* `:decimal_final_balance` balance after the transactions, but as float\n* `:initial_balance` balance before the transactions (integer)\n* `:decimal_initial_balance` balance after the transactions, but as float","ref":"Numscriptex.Balance.html#t:t/0-fields"},{"type":"module","title":"Numscriptex.Builder","doc":"`Numscriptex.Builder` makes it possible to build Numscripts dynamically within your application.","ref":"Numscriptex.Builder.html"},{"type":"function","title":"Numscriptex.Builder.build/1","doc":"Receives a map with the metadata necessary to build your numscript.\n\n```elixir\niex> metadata = %{\n...>   splits: [\n...>     %{\n...>       type: :fixed,\n...>       amount: 500,\n...>       asset: \"BRL/2\",\n...>       account: \"some:destination\"\n...>     }\n...>   ]\n...> }\n...>\n...> Numscriptex.Builder.build(metadata)\n{:ok, %{script: \"send [BRL/2 500] (\n  source = @user\n  destination = @some:destination\n)\n\"}}\n```\n\nIf you want to learn more about this feature you can check its guide [here](https://hexdocs.pm/numscriptex/builder-introduction.html)","ref":"Numscriptex.Builder.html#build/1"},{"type":"type","title":"Numscriptex.Builder.fixed_split/0","doc":"","ref":"Numscriptex.Builder.html#t:fixed_split/0"},{"type":"type","title":"Numscriptex.Builder.metadata/0","doc":"","ref":"Numscriptex.Builder.html#t:metadata/0"},{"type":"type","title":"Numscriptex.Builder.percent_split/0","doc":"","ref":"Numscriptex.Builder.html#t:percent_split/0"},{"type":"module","title":"Numscriptex.CheckLog","doc":"After you check your numscript you might get a variety of logs even if it is valid,\n`Numscriptex.CheckLog` is responsible for standardize these logs.","ref":"Numscriptex.CheckLog.html"},{"type":"function","title":"Numscriptex.CheckLog.from_map/1","doc":"Get a map with log data about a checked numscript.\n\n```elixir\niex> map = %{\n...>   character: 10,\n...>   level:  :warning,\n...>   line: 1,\n...>   message: \"warning message\"\n...> }\n...>\n...> Numscriptex.CheckLog.from_map(map)\n%Numscriptex.CheckLog{\n  character: 10,\n  level:  :warning,\n  line: 1,\n  message: \"warning message\"\n}\n```","ref":"Numscriptex.CheckLog.html#from_map/1"},{"type":"type","title":"Numscriptex.CheckLog.log_levels/0","doc":"","ref":"Numscriptex.CheckLog.html#t:log_levels/0"},{"type":"type","title":"Numscriptex.CheckLog.t/0","doc":"Type that represents `Numscriptex.CheckLog` struct.","ref":"Numscriptex.CheckLog.html#t:t/0"},{"type":"type","title":"Fields - Numscriptex.CheckLog.t/0","doc":"* `:character` the character position where the log occurred\n* `:level` the log level\n* `:line` the line where the log occur\n* `:message` the log message","ref":"Numscriptex.CheckLog.html#t:t/0-fields"},{"type":"module","title":"Numscriptex.Posting","doc":"`Numscriptex.Postings` represents a financial transaction made with [Numscript](https://docs.formance.com/numscript/)","ref":"Numscriptex.Posting.html"},{"type":"function","title":"Numscriptex.Posting.from_list/1","doc":"","ref":"Numscriptex.Posting.html#from_list/1"},{"type":"function","title":"Numscriptex.Posting.from_map/1","doc":"","ref":"Numscriptex.Posting.html#from_map/1"},{"type":"type","title":"Numscriptex.Posting.t/0","doc":"Type that represents `Numscriptex.Posting` struct.","ref":"Numscriptex.Posting.html#t:t/0"},{"type":"type","title":"Fields - Numscriptex.Posting.t/0","doc":"* `:source` account whose the money came from\n* `:asset` the asset were the transaction was made\n* `:destination` account whose the money will go to\n* `:amount` amount of money transferred (integer)\n* `:decimal_amount` amount of money transferred, but as float","ref":"Numscriptex.Posting.html#t:t/0-fields"},{"type":"module","title":"Numscriptex.Run","doc":"A Numscript needs some other data aside from the script itself to run correctly,\nand `Numscriptex.Run` solves this problem. If you want to know what exactly these\nadditional fields are, you can learn more on [Numscript Playground](https://playground.numscript.org/?template=simple-send) and the [Numscript Docs](https://docs.formance.com/numscript/).","ref":"Numscriptex.Run.html"},{"type":"function","title":"Numscriptex.Run.new/0","doc":"Creates a new `Numscriptex.Run{}` struct.\n\n```elixir\niex>  Numscriptex.Run.new()\n%Numscriptex.Run{\n  variables: %{},\n  balances: %{},\n  metadata: %{}\n}\n```","ref":"Numscriptex.Run.html#new/0"},{"type":"function","title":"Numscriptex.Run.put/3","doc":"Puts the chosen value under the field key on the `Numscriptex.Run` struct as\nlong both are valid.\n\n```elixir\niex> struct = Numscriptex.Run.new()\n...> balances =  %{\"foo\" => %{\"USD/2\" => 500, \"EUR/2\" => 300}}\n...>\n...> Numscriptex.Run.put(struct, :balances, balances)\n{:ok,\n  %Numscriptex.Run{\n    balances: %{\"foo\" => %{\"USD/2\" => 500, \"EUR/2\" => 300}},\n    variables: %{},\n    metadata: %{}\n  }\n}\n```\nIf the value or the field key are invalid, `put/3` will return a 3 element\nerror tuple. Ex:\n\n```elixir\niex> struct = Numscriptex.Run.new()\n...>\n...> Numscriptex.Run.put(struct, :invalid_field, %{})\n{:error, :invalid_field, %{details: \"The field 'invalid_field' does not exists.\"}}\n```","ref":"Numscriptex.Run.html#put/3"},{"type":"function","title":"Numscriptex.Run.put!/3","doc":"Same as `put/3`, but raises an exception if any argument is invalid.\n\n```elixir\niex> struct = Numscriptex.Run.new()\n...> balances =  [%{\"foo\" => %{\"USD/2\" => 500, \"EUR/2\" => 300}}]\n...>\n...> Numscriptex.Run.put!(struct, :balances, balances)\n** (ArgumentError) Argument `value` must be a map.\n```","ref":"Numscriptex.Run.html#put!/3"},{"type":"type","title":"Numscriptex.Run.t/0","doc":"Type that represents `Numscriptex.Run` struct.","ref":"Numscriptex.Run.html#t:t/0"},{"type":"type","title":"Fields - Numscriptex.Run.t/0","doc":"* `:balances` a map with account's assets balances\n* `:metadata` [metada variables](https://docs.formance.com/numscript/reference/metadata)\n* `:variables` [variables](https://docs.formance.com/numscript/reference/variables) used inside the script","ref":"Numscriptex.Run.html#t:t/0-fields"},{"type":"module","title":"Numscriptex.Utilities","doc":"`Numscriptex.Utilities` module contain reusable code that are common in more than one module\nof this library.","ref":"Numscriptex.Utilities.html"},{"type":"function","title":"Numscriptex.Utilities.decimal_places_from_asset/2","doc":"Gets the decimal places from an asset if has any (e.g. decimal places for \"USD/2\" would be 2).\nYou can choose a default value (the second argument), but if you don't choose any the default will be 2.\n\n```elixir\niex> Utilities.decimal_places_from_asset(\"USD/2\")\n2\n\niex> Utilities.decimal_places_from_asset(\"USD/4\")\n4\n\niex> Utilities.decimal_places_from_asset(\"USD\")\n2\n\niex> Utilities.decimal_places_from_asset(\"USD\", 3)\n3\n```","ref":"Numscriptex.Utilities.html#decimal_places_from_asset/2"},{"type":"function","title":"Numscriptex.Utilities.integer_to_decimal/2","doc":"Converts an integer value to float\n\n```elixir\niex> Utilities.integer_to_decimal(1000, 2)\n10.0\n\niex> Utilities.integer_to_decimal(1000, 3)\n1.0\n\niex> Utilities.integer_to_decimal(1000, 4)\n0.1\n\niex> Utilities.integer_to_decimal(1000, 5)\n0.01\n```","ref":"Numscriptex.Utilities.html#integer_to_decimal/2"},{"type":"function","title":"Numscriptex.Utilities.normalize_keys/2","doc":"Convert map keys to the desired type. Can go from string to atom and vice versa.\n\n```elixir\niex> map = %{\"foo\" => 100}\n...> Utilities.normalize_keys(map, :atom)\n%{foo: 100}\n\niex> map = %{foo: 100}\n...> Utilities.normalize_keys(map, :string)\n%{\"foo\" => 100}\n```","ref":"Numscriptex.Utilities.html#normalize_keys/2"},{"type":"extras","title":"Readme","doc":"# NumscriptEx\n[![Hex Version](https://img.shields.io/hexpm/v/numscriptex.svg)](https://hex.pm/packages/numscriptex)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/numscriptex/)\n\nNumscriptEx is a library that allows its users to check and run Numscripts in Elixir. If this is your first time hearing about\nNumscripts, here is a quick explanation:\n\n[Numscript](https://docs.formance.com/numscript/) is a DSL made by [Formance](https://www.formance.com/)\nthat simplifies complex financial transactions with scripts that are easy to read,\nso you don't need a big, complex and error-prone codebase to deal with your finances.\n\nYou can see and execute some examples at the [Numscript Playground](https://playground.numscript.org/?template=simple-send).","ref":"readme.html"},{"type":"extras","title":"Installation - Readme","doc":"You will just need to add `:numscriptex` as a dependency on your `mix.exs`, and run the `mix deps.get` command:\n\n```elixir\ndef deps do\n  [\n    {:numscriptex, \"~> 0.2.5\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Configuration - Readme","doc":"NumscriptEx needs some external assets ([Numscript-WASM](https://github.com/PagoPlus/numscript-wasm)),\nand you can override the default version.\n\nAvailable configurations:\n- `:version` the binary release version to use (see [numscript-wasm releases](https://github.com/PagoPlus/numscript-wasm/releases/)).\n- `:retries` number of times to retry the download in case of a network failure.\n- `:binary_path` where the WASM binary file will be downloaded\n\nEx:\n```elixir\nconfig :numscriptex,\n       binary_path: :numscriptex |> :code.priv_dir() |> Path.join(\"numscript.wasm\")\n       version: \"0.0.2\",\n       retries: 3\n```\nThese above are the default values.","ref":"readme.html#configuration"},{"type":"extras","title":"Usage - Readme","doc":"You can build Numscripts dynamically with `Numscriptex.Builder.build/1`, check if your script is valid with `Numscriptex.check/1`,\nand last but not least, you can run your script with `Numscriptex.run/2`.\n\nYou can read more about the `Numscriptex.Builder` module and how to use it on its [guide](https://hexdocs.pm/numscriptex/builder-introduction.html)\n\nAnd before introducing the other two functions, you will need to know what is the `Numscriptex.Run` struct.","ref":"readme.html#usage"},{"type":"extras","title":"Numscriptex.Run - Readme","doc":"A numscript needs some other data aside the script itself to run correctly, and\n`Numscriptex.Run` solves this problem.\n\nIf you want to know what exactly these additional data are, you can see the\n[Numscript Playground](https://playground.numscript.org/?template=simple-send) for examples.\n\nThe abstraction is made by creating a struct:\n```elixir\niex>  %Numscriptex.Run{\n...>    balances: %{},\n...>    metadata: %{},\n...>    variables: %{}\n...>  }\n```\nWhere:\n- `:balances` a map with the account's assets balances.\n- `:metadata` [metada variables](https://docs.formance.com/numscript/reference/metadata);\n- `:variables` [variables](https://docs.formance.com/numscript/reference/variables) used inside the script.\n\nAnd to create a new struct, you can use the `Numscriptex.Run.put/3` or `Numscriptex.Run.put!/3` functions. Ex:\n```elixir\niex>  variables = %{\"order\" => \"orders:2345\"}\n...>  balances = %{\"orders:2345\" => %{\"USD/2\" => 1000}}\n...>  metadata = %{\n...>    \"merchants:1234\" => %{\"commission\" => \"15%\"},\n...>    \"orders:2345\" => %{\"merchant\" => \"merchants:1234\"}\n...>  }\n...>\n...>  Numscriptex.Run.new()\n...>  |> Numscriptex.Run.put!(:balances, balances)\n...>  |> Numscriptex.Run.put!(:metadata, metadata)\n...>  |> Numscriptex.Run.put!(:variables, variables)\n```\n\nWill return:\n\n```elixir\niex> %Numscriptex.Run{\n...>   variables: %{\"orders:2345\" => %{\"USD/2\" => 1000}},\n...>   balances: %{\"order\" => \"orders:2345\"},\n...>   metadata: %{\n...>     \"merchants:1234\" => %{\"commission\" => \"15%\"},\n...>     \"orders:2345\" => %{\"merchant\" => \"merchants:1234\"}\n...>   }\n...> }\n```\n\nKindly reminder: you will always need a valid `Numscriptex.Run` struct to successfully execute your scripts.","ref":"readme.html#numscriptex-run"},{"type":"extras","title":"Check - Readme","doc":"To use `Numscriptex.check/1` you just have to pass your numscript as it's argument. Ex:\n\n```elixir\niex>  \"tmp/script.num\"\n...>  |> File.read!()\n...>  |> Numscriptex.check()\n{:ok, %{script: script}\n```\n\nYou don´t need to necessarily read from a file, as long as it is a string it's fine.\n\nSometimes, even if your script is valid, it could also return some warnings, infos or hints inside the map.\nEx:\n```elixir\niex> {:ok, %{\n...>     script: \"your numscript here\",\n...>     warnings: [\n...>             %CheckLog{\n...>               character: 10,\n...>               level: :warning,\n...>               line: 1,\n...>               message: \"warning message\"\n...>             }\n...>           ],\n...>     hints: [\n...>             %CheckLog{\n...>               character: 2,\n...>               level: :hint,\n...>               line: 7,\n...>               message: \"hint message\"\n...>             }\n...>           ],\n...>     infos: [\n...>             %CheckLog{\n...>               character: 9,\n...>               level: :info,\n...>               line: 14,\n...>               message: \"info message\"\n...>             }\n...>           ]\n...>   }\n...> }\n```\nThe `:script` is the only field that will always return if your script is valid, the other three are optional.","ref":"readme.html#check"},{"type":"extras","title":"Run - Readme","doc":"To use `Numscriptex.run/2` your first argument must be your script (the same you used in `Numscriptex.check/1`), and the second must be the `%Numscriptex.Run{}` struct. Ex:\n\n```elixir\niex>  Numscriptex.run(script, struct)\n{:ok, result}\n```\n\nWhere result will be something like this:\n```elixir\niex> %{\n...>   postings: [\n...>           %Numscriptex.Posting{\n...>             amount: 100,\n...>             decimal_amount: 1.0,\n...>             asset: \"USD/2\",\n...>             destination: \"bar\",\n...>             source: \"foo\"\n...>           }\n...>         ],\n...>   balances: [\n...>           %Numscriptex.Balance{\n...>             account: \"foo\",\n...>             asset: \"EUR/2\",\n...>             final_balance: 300,\n...>             initial_balance: 300,\n...>             decimal_final_balance: 3.0,\n...>             decimal_initial_balance: 3.0,\n...>           },\n...>           %Numscriptex.Balance{\n...>             account: \"foo\",\n...>             asset: \"USD/2\",\n...>             final_balance: 400,\n...>             initial_balance: 500,\n...>             decimal_final_balance: 4.0,\n...>             decimal_initial_balance: 5.0,\n...>           },\n...>           %Numscriptex.Balance{\n...>             account: \"bar\",\n...>             asset: \"USD/2\",\n...>             final_balance: 100,\n...>             initial_balance: 0,\n...>             decimal_final_balance: 1.0,\n...>             decimal_initial_balance: 0.0,\n...>           }\n...>         ],\n...>   accountMeta: %{}\n...>   txMeta: %{}\n...> }\n```","ref":"readme.html#run"},{"type":"extras","title":"License - Readme","doc":"Copyright (c) 2025 MedFlow\n\nThis library is MIT licensed. See the [LICENSE](https://github.com/PagoPlus/numscriptex/blob/main/LICENSE) for details.","ref":"readme.html#license"},{"type":"extras","title":"Building Numscripts","doc":"# Builder\nThis is a feature that makes it possible to build numscripts dynamically within your application.","ref":"builder-introduction.html"},{"type":"extras","title":"Usage - Building Numscripts","doc":"You just need to call `Numscriptex.Builder.build/1` with the metadata necessary to build your numscript.","ref":"builder-introduction.html#usage"},{"type":"extras","title":"Metadata - Building Numscripts","doc":"Metadata is the sole argument you need to use the `build/1` function, wich has the following fields:\n\nRequired fields:\n- splits: a list of metadata needed to build numscripts (explained below)\n\nOptional fields:\n- percent_asset: monetary asset (BRL, USD, EUR, and etc.) to use when build numscripts using percentage values.\n- remaining_to: where the rest of the money will go after the transaction is made\n  - Only needed if you want to designate a specific destination. The default is [remaining kept](https://docs.formance.com/numscript/reference/destinations#allocation-destinations) wich makes the remainder go back to the source.\n\nAbout the \"list of metadata\" mentioned on the `split` field above:\n\nRequired fields:\n- account: account whose the money will go to\n- amount: amount of money transferred. Be aware that this field only accepts integer values, so if you want to send $5 your amount field value should be 500.\n- type: the amount type, accepts two values:\n  - fixed: treats the `amount` field as a number\n  - percent: treats the `amount` field as a percentage value\n\nOptional fields:\nAlso accepts `aplits` and `remaining_to` fields, and a `asset` field:\n\n- splits: in this case, this field is only neccessary if you want to build [nested destinations](https://docs.formance.com/numscript/reference/destinations#nested-destinations), and therefore it is only valid if the `type` field value is `:percent`\n- asset: also the monetary asset, but in this case is only needed on `:fixed` types\n\nThe `remaining_to` field continues with the same behaviour as above.","ref":"builder-introduction.html#metadata"},{"type":"extras","title":"Example - Building Numscripts","doc":"With the metadata type above in mind, let's build a simple numscript.\n```elixir\n# metadata:\n%{\n  splits: [\n    %{\n      type: :percent,\n      amount: 20,\n      account: \"some:destination\"\n    }\n  ],\n  remaining_to: \"another:destination\",\n  percent_asset: \"USD\"\n}\n```\nWich will generate the following numscript:\n```\nsend [USD *] (\n  source = @user\n  destination = {\n    20% to @some:destination\n    remaining to @another:destination\n  }\n)\n```\nIf the `:remaining_to` field were not defined, the remaining would be kept:\n```\nsend [USD *] (\n  source = @user\n  destination = {\n    20% to @some:destination\n    remaining kept\n  }\n)\n```\n\nP.S.: The percent amount types will always be executed first, so we can assure that\nall of the percentages are based off of the initial value. This feature is still in\ndevelopment as we speak, but soon we will have more flexibility in those regards.","ref":"builder-introduction.html#example"},{"type":"extras","title":"Examples - Building Numscripts","doc":"","ref":"builder-introduction.html#examples"},{"type":"extras","title":"Example 1 - Building Numscripts","doc":"Both fixed and percent types:\n```elixir\n%{\n  splits: [\n    %{\n      type: :fixed,\n      amount: 500,\n      asset: \"USD\",\n      account: \"some:destination:a\"\n    },\n    %{\n      type: :percent,\n      amount: 20,\n      account: \"some:destination:b\"\n    }\n  ],\n  remaining_to: \"some:destination:c\",\n  percent_asset: \"USD\"\n}\n```\nWill generate:\n```\nsend [USD *] (\n  source = @user\n  destination = {\n    20% to @some:destination:b\n    remaining kept\n  }\n)\n\nsend [USD 500] (\n  source = @user\n  destination = @some:destination:a\n)\n```","ref":"builder-introduction.html#example-1"},{"type":"extras","title":"Example 2 - Building Numscripts","doc":"Nested percent-type, plus fixed\n```elixir\n%{\n  splits: [\n    %{\n      type: :fixed,\n      amount: 1250,\n      asset: \"USD\",\n      account: \"some:destination:a\"\n    },\n    %{\n      type: :percent,\n      amount: 30,\n      remaining_to: \"remaining:destination:b\",\n      splits: [\n        %{\n          type: :percent,\n          amount: 40,\n          account: \"some:destination:c\"\n        },\n        %{\n          type: :percent,\n          amount: 20,\n          account: \"some:destination:b\"\n        }\n      ]\n    }\n  ],\n  remaining_to: \"remaining:destination:a\",\n  percent_asset: \"USD\"\n}\n```\nWill generate:\n```\nsend [USD *] (\n  source = @user\n  destination = {\n    30% to {\n      40% to @some:destination:c\n      20% to @some:destination:b\n      remaining to remaining:destination:b\n    }\n    remaining to remaining:destination:a\n  }\n)\n\nsend [USD 1250] (\n  source = @user\n  destination = @some:destination:a\n)\n```\nNote that the `asset` field inside of nested destinations will be ignored, given that\nthey all go under the same `send` scope.","ref":"builder-introduction.html#example-2"},{"type":"extras","title":"Example 3 - Building Numscripts","doc":"Nested percent-type, plus fixed, and with fixed-type within nests\n```elixir\n%{\n  splits: [\n    %{\n      type: :fixed,\n      amount: 1050,\n      asset: \"EUR\",\n      account: \"some:destination:a\"\n    },\n    %{\n      type: :percent,\n      amount: 20,\n      account: \"some:destination:b\"\n    },\n    %{\n      type: :percent,\n      amount: 30,\n      remaining_to: \"remaining:destination:b\",\n      splits: [\n        %{\n          type: :fixed,\n          amount: 1250,\n          asset: \"USD\",\n          account: \"some:destination:c\"\n        },\n        %{\n          type: :percent,\n          amount: 15,\n          account: \"some:destination:e\"\n        },\n        %{\n          type: :percent,\n          amount: 20,\n          account: \"some:destination:d\"\n        },\n        %{\n          type: :percent,\n          amount: 50,\n          remaining_to: \"remaining:destination:c\",\n          splits: [\n            %{\n              type: :fixed,\n              amount: 500,\n              asset: \"BRL\",\n              account: \"some:destination:b\"\n            },\n            %{\n              type: :percent,\n              amount: 20,\n              account: \"some:destination:a\"\n            },\n            %{\n              type: :percent,\n              amount: 27,\n              account: \"some:destination:a\"\n            }\n          ]\n        }\n      ]\n    }\n  ],\n  remaining_to: \"remaining:destination:a\",\n  percent_asset: \"EUR\"\n}\n```\nWill generate:\n```\nsend [EUR *] (\n  source = @user\n  destination = {\n    20% to @some:destination:b\n    30% to {\n      15% to @some:destination:e\n      20% to @some:destination:d\n      50% to {\n        20% to @some:destination:a\n        27% to @some:destination:a\n        remaining to @remaining:destination:c\n      }\n      remaining to @remaining:destination:b\n    }\n    remaining to @remaining:destination:a\n  }\n)\n\nsend [USD 1250] (\n  source = @user\n  destination = @some:destination:c\n)\n\nsend [BRL 500] (\n  source = @user\n  destination = @some:destination:b\n)\n\nsend [EUR 1050] (\n  source = @user\n  destination = @some:destination:a\n)\n```\nNote that the `fixed` types in nested `splits` (i.e. `splits` fields inside of `splits` fields) will be popped out and still\nbe built after the `percent` types.","ref":"builder-introduction.html#example-3"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}